@@ -1,6 +1,8 @@
 extern crate odds;
 extern crate nodrop;
 
+use std::cmp;
+use std::io;
 use std::iter;
 use std::mem;
 use std::ptr;
@@ -33,7 +35,7 @@ unsafe fn new_array<A: Array>() -> A {
 
 /// A vector with a fixed capacity.
 ///
-/// The **ArrayVec** is a vector backed by a fixed size array. It keeps track of
+/// The `ArrayVec` is a vector backed by a fixed size array. It keeps track of
 /// the number of initialized elements.
 ///
 /// The vector is a contiguous value that you can store directly on the stack
@@ -61,12 +63,10 @@ impl<A: Array> Drop for ArrayVec<A> {
 }
 
 impl<A: Array> ArrayVec<A> {
-    /// Create a new empty **ArrayVec**.
+    /// Create a new empty `ArrayVec`.
     ///
     /// Capacity is inferred from the type parameter.
     ///
-    /// ## Examples
-    ///
     /// ```
     /// use arrayvec::ArrayVec;
     ///
@@ -82,9 +82,8 @@ impl<A: Array> ArrayVec<A> {
         }
     }
 
-    /// Return the number of elements in the **ArrayVec**.
+    /// Return the number of elements in the `ArrayVec`.
     ///
-    /// ## Examples
     /// ```
     /// use arrayvec::ArrayVec;
     ///
@@ -95,9 +94,8 @@ impl<A: Array> ArrayVec<A> {
     #[inline]
     pub fn len(&self) -> usize { self.len.to_usize() }
 
-    /// Return the capacity of the **ArrayVec**.
+    /// Return the capacity of the `ArrayVec`.
     ///
-    /// ## Examples
     /// ```
     /// use arrayvec::ArrayVec;
     ///
@@ -107,12 +105,11 @@ impl<A: Array> ArrayVec<A> {
     #[inline]
     pub fn capacity(&self) -> usize { A::capacity() }
 
-    /// Push **element** to the end of the vector.
+    /// Push `element` to the end of the vector.
     ///
-    /// Return **None** if the push succeeds, or and return **Some(** *element* **)**
+    /// Return `None` if the push succeeds, or and return `Some(` *element* `)`
     /// if the vector is full.
     ///
-    /// ## Examples
     /// ```
     /// use arrayvec::ArrayVec;
     ///
@@ -140,9 +137,8 @@ impl<A: Array> ArrayVec<A> {
 
     /// Remove the last element in the vector.
     ///
-    /// Return **Some(** *element* **)** if the vector is non-empty, else **None**.
+    /// Return `Some(` *element* `)` if the vector is non-empty, else `None`.
     ///
-    /// ## Examples
     /// ```
     /// use arrayvec::ArrayVec;
     ///
@@ -164,13 +160,12 @@ impl<A: Array> ArrayVec<A> {
         }
     }
 
-    /// Insert **element** in position **index**.
+    /// Insert `element` in position `index`.
     ///
-    /// Shift up all elements after **index**. If any is pushed out, it is returned.
+    /// Shift up all elements after `index`. If any is pushed out, it is returned.
     ///
-    /// Return None if no element is shifted out.
+    /// Return `None` if no element is shifted out.
     ///
-    /// ## Examples
     /// ```
     /// use arrayvec::ArrayVec;
     ///
@@ -188,8 +183,7 @@ impl<A: Array> ArrayVec<A> {
             return Some(element);
         }
         let mut ret = None;
-        let old_len = self.len();
-        if old_len == self.capacity() {
+        if self.len() == self.capacity() {
             ret = self.pop();
         }
         let len = self.len();
@@ -198,26 +192,25 @@ impl<A: Array> ArrayVec<A> {
         unsafe { // infallible
             // The spot to put the new value
             {
-                let p = self.as_mut_ptr().offset(index as isize);
+                let p = self.get_unchecked_mut(index) as *mut _;
                 // Shift everything over to make space. (Duplicating the
                 // `index`th element into two consecutive places.)
-                ptr::copy(&*p, p.offset(1), len - index);
+                ptr::copy(p, p.offset(1), len - index);
                 // Write it in, overwriting the first copy of the `index`th
                 // element.
-                ptr::write(&mut *p, element);
+                ptr::write(p, element);
             }
             self.set_len(len + 1);
         }
         ret
     }
 
-    /// Remove the element at **index** and swap the last element into its place.
+    /// Remove the element at `index` and swap the last element into its place.
     ///
     /// This operation is O(1).
     ///
-    /// Return **Some(** *element* **)** if the index is in bounds, else **None**.
+    /// Return `Some(` *element* `)` if the index is in bounds, else `None`.
     ///
-    /// ## Examples
     /// ```
     /// use arrayvec::ArrayVec;
     ///
@@ -237,11 +230,10 @@ impl<A: Array> ArrayVec<A> {
         self.pop()
     }
 
-    /// Remove the element at **index** and shift down the following elements.
+    /// Remove the element at `index` and shift down the following elements.
     ///
-    /// Return **Some(** *element* **)** if the index is in bounds, else **None**.
+    /// Return `Some(` *element* `)` if the index is in bounds, else `None`.
     ///
-    /// ## Examples
     /// ```
     /// use arrayvec::ArrayVec;
     ///
@@ -267,9 +259,9 @@ impl<A: Array> ArrayVec<A> {
 
     /// Set the vector's length without dropping or moving out elements
     ///
-    /// May panic if **length** is greater than the capacity.
+    /// May panic if `length` is greater than the capacity.
     ///
-    /// This function is **unsafe** because it changes the notion of the
+    /// This function is `unsafe` because it changes the notion of the
     /// number of “valid” elements in the vector. Use with care.
     #[inline]
     pub unsafe fn set_len(&mut self, length: usize) {
@@ -285,13 +277,9 @@ impl<A: Array> ArrayVec<A> {
     /// Note: It is unspecified how many elements are removed from the vector,
     /// if the `Drain` value is leaked.
     ///
-    /// # Panics
-    ///
-    /// Panics if the starting point is greater than the end point or if
+    /// **Panics** if the starting point is greater than the end point or if
     /// the end point is greater than the length of the vector.
     ///
-    /// # Examples
-    ///
     /// ```
     /// use arrayvec::ArrayVec;
     ///
@@ -331,10 +319,10 @@ impl<A: Array> ArrayVec<A> {
 
     /// Return the inner fixed size array, if it is full to its capacity.
     ///
-    /// Return an **Ok** value with the array if length equals capacity,
-    /// return an **Err** with self otherwise.
+    /// Return an `Ok` value with the array if length equals capacity,
+    /// return an `Err` with self otherwise.
     ///
-    /// **Note:** This function may incur unproportionally large overhead
+    /// `Note:` This function may incur unproportionally large overhead
     /// to move the array out, its performance is not optimal.
     pub fn into_inner(self) -> Result<A, Self> {
         if self.len() < self.capacity() {
@@ -369,9 +357,8 @@ impl<A: Array> DerefMut for ArrayVec<A> {
     }
 }
 
-/// Create an **ArrayVec** from an array.
+/// Create an `ArrayVec` from an array.
 ///
-/// ## Examples
 /// ```
 /// use arrayvec::ArrayVec;
 ///
@@ -386,9 +373,7 @@ impl<A: Array> From<A> for ArrayVec<A> {
 }
 
 
-/// Iterate the **ArrayVec** with references to each element.
-///
-/// ## Examples
+/// Iterate the `ArrayVec` with references to each element.
 ///
 /// ```
 /// use arrayvec::ArrayVec;
@@ -405,9 +390,7 @@ impl<'a, A: Array> IntoIterator for &'a ArrayVec<A> {
     fn into_iter(self) -> Self::IntoIter { self.iter() }
 }
 
-/// Iterate the **ArrayVec** with mutable references to each element.
-///
-/// ## Examples
+/// Iterate the `ArrayVec` with mutable references to each element.
 ///
 /// ```
 /// use arrayvec::ArrayVec;
@@ -424,12 +407,10 @@ impl<'a, A: Array> IntoIterator for &'a mut ArrayVec<A> {
     fn into_iter(self) -> Self::IntoIter { self.iter_mut() }
 }
 
-/// Iterate the **ArrayVec** with each element by value.
+/// Iterate the `ArrayVec` with each element by value.
 ///
 /// The vector is consumed by this operation.
 ///
-/// ## Examples
-///
 /// ```
 /// use arrayvec::ArrayVec;
 ///
@@ -446,7 +427,7 @@ impl<A: Array> IntoIterator for ArrayVec<A> {
 }
 
 
-/// By-value iterator for **ArrayVec**.
+/// By-value iterator for `ArrayVec`.
 pub struct IntoIter<A: Array> {
     index: A::Index,
     v: ArrayVec<A>,
@@ -507,7 +488,7 @@ impl<A: Array> Drop for IntoIter<A> {
     }
 }
 
-/// A draining iterator for **ArrayVec**.
+/// A draining iterator for `ArrayVec`.
 pub struct Drain<'a, A> 
     where A: Array,
           A::Item: 'a,
@@ -586,7 +567,7 @@ impl<'a, A: Array> Drop for Drain<'a, A>
 
 
 
-/// Extend the **ArrayVec** with an iterator.
+/// Extend the `ArrayVec` with an iterator.
 /// 
 /// Does not extract more items than there is space for. No error
 /// occurs if there are more iterator elements.
@@ -599,7 +580,7 @@ impl<A: Array> Extend<A::Item> for ArrayVec<A> {
     }
 }
 
-/// Create an **ArrayVec** from an iterator.
+/// Create an `ArrayVec` from an iterator.
 /// 
 /// Does not extract more items than there is space for. No error
 /// occurs if there are more iterator elements.
@@ -656,3 +637,59 @@ impl<A: Array> AsMut<[A::Item]> for ArrayVec<A> {
 impl<A: Array> fmt::Debug for ArrayVec<A> where A::Item: fmt::Debug {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result { (**self).fmt(f) }
 }
+
+impl<A: Array> Default for ArrayVec<A> {
+    fn default() -> ArrayVec<A> {
+        ArrayVec::new()
+    }
+}
+
+impl<A: Array> PartialOrd for ArrayVec<A> where A::Item: PartialOrd {
+    #[inline]
+    fn partial_cmp(&self, other: &ArrayVec<A>) -> Option<cmp::Ordering> {
+        (**self).partial_cmp(other)
+    }
+
+    #[inline]
+    fn lt(&self, other: &Self) -> bool {
+        (**self).lt(other)
+    }
+
+    #[inline]
+    fn le(&self, other: &Self) -> bool {
+        (**self).le(other)
+    }
+
+    #[inline]
+    fn ge(&self, other: &Self) -> bool {
+        (**self).ge(other)
+    }
+
+    #[inline]
+    fn gt(&self, other: &Self) -> bool {
+        (**self).gt(other)
+    }
+}
+
+impl<A: Array> Ord for ArrayVec<A> where A::Item: Ord {
+    fn cmp(&self, other: &ArrayVec<A>) -> cmp::Ordering {
+        (**self).cmp(other)
+    }
+}
+
+/// `Write` appends written data to the end of the vector.
+impl<A: Array<Item=u8>> io::Write for ArrayVec<A> {
+    fn write(&mut self, data: &[u8]) -> io::Result<usize> {
+        unsafe {
+            let len = self.len();
+            let mut tail = slice::from_raw_parts_mut(self.get_unchecked_mut(len),
+                                                     A::capacity() - len);
+            let result = tail.write(data);
+            if let Ok(written) = result {
+                self.set_len(len + written);
+            }
+            result
+        }
+    }
+    fn flush(&mut self) -> io::Result<()> { Ok(()) }
+}
